package brocchini.diy.mock;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;

/**
 * Handles method calls.
 * 
 * @author Mozart Brocchini
 * @since 11/2011
 */
public class CallHandler implements InvocationHandler {

	final Map<String, Object> interceptions;

	/** Delegation of Interfaces mapping to implementation objects */
	final Map<Class<?>, Object> delegation;

	public CallHandler(Map<String, Object> interceptions, Map<Class<?>, Object> delegation) {
		super();
		this.interceptions = interceptions;
		this.delegation = delegation;
	}

	public <T> CallHandler intercept(String method, T value) {
		interceptions.put(method, value);
		return this;
	}

	public <T> CallHandler delegate(Class<T> clazz, T value) {
		delegation.put(clazz, value);
		return this;
	}
	
	@Override
	public Object invoke(Object proxy, Method method, Object[] args)
			throws Throwable {

		// Try interceptions first
		if (interceptions.containsKey(method.getName())) {
			return interceptions.get(method.getName());
		}

		// Now try delegations
		Class<?> clientClass = method.getDeclaringClass();
		for (Class<?> type : delegation.keySet()) {
			if (clientClass.isAssignableFrom(type)) {
				try {
					return method.invoke(delegation.get(type), args);
				} catch (InvocationTargetException e) {
					throw e.getTargetException();
				}
			}
		}
		
		// Finally return null for non delegated non intercepted methods
		System.out.println(method.getName()
				+ " was not delegated. returning null");
		return null;
	}
}