package brocchini.diy.mock;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;
import java.util.HashMap;
import java.util.Map;

/**
 * Minimalistic mocking framework.
 * 
 * @author Mozart Brocchini
 * @since 11/2011
 * 
 */
public class DIYMock {

	/** Intercept a method to return any value */
	final Map<String, Object> interceptions= new HashMap<String, Object>();

	/** Delegation of Interfaces mapping to implementation objects */
	final Map<Class<?>, Object> delegation=new HashMap<Class<?>, Object>();
	
	public InvocationHandler handler = new CallHandler(interceptions, delegation);

	@SuppressWarnings("unchecked")
	public <T> T mock(Class<T> clazz) {
		return (T) Proxy.newProxyInstance(this.getClass().getClassLoader(),
				new Class[] { clazz }, handler);
	}

	public <T> DIYMock intercept(String method, T value) {
		interceptions.put(method, value);
		return this;
	}

	public <T> DIYMock delegate(Class<T> clazz, T value) {
		delegation.put(clazz, value);
		return this;
	}
}
