package brocchini.diy.mock;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertNotNull;

import org.junit.Test;

import brocchini.Car;
import brocchini.diy.mock.DIYMock;

public class DIYMockTest {

	DIYMock mocker;
	static String mockedResult = "proxy roll!";

	@Test
	public void test() {
		mocker = new DIYMock();
		
		Car car = new Car() {
			@Override
			public String drive() {
				return "rolling...";
			}

			@Override
			public String park() {
				return "stalled.";
			}
		};

		// Original
		assertTrue("rolling...".equals( car.drive()));
		assertNotNull(car.park());	
		
		// Create a mock, for now all methods return null
		Car carMock = mocker.mock(Car.class);
		
		// Intercept method .drive() and make it return 'proxy roll!'
	    mocker.intercept("drive", mockedResult);

	    // Confirm the result of method drive 
		assertTrue(mockedResult.equals(carMock.drive()));
		
		// Method park returns null because it was not intercepted or delegated.
		assertNull(carMock.park());
		
		// Delegate method calls to the object car
		mocker.delegate(Car.class, car);
		
		// Now park() returns 'stalled.' because it was delegated to car.park()
		assertTrue("stalled.".equals(carMock.park()));
		
		// The interception prevails over delegation
		assertTrue(mockedResult.equals(carMock.drive()));
		
		
	}
}
